@@ -31,12 +31,9 @@
 // cumulative mid/side normalization factor (1/sqrt(2))*(1/sqrt(2))
 #define STEREO_NORM_FACTOR 0.5
 
-#define INITIAL_DITHER_SEED 0x12345000
-
 static uint8_t panningMode;
-static int32_t randSeed = INITIAL_DITHER_SEED, stereoSeparation = 100;
-static double *dMixBufferL, *dMixBufferR;
-static double dPrngStateL, dPrngStateR, dSideFactor;
+static int32_t stereoSeparation = 100;
+static double *dMixBufferL, *dMixBufferR, dSideFactor;
 static SDL_AudioDeviceID dev;
 
 audio_t audio; // globalized
@@ -130,27 +127,11 @@ void unlockAudio(void)
 	audio.locked = false;
 }
 
-void resetAudioDithering(void)
-{
-	randSeed = INITIAL_DITHER_SEED;
-	dPrngStateL = 0.0;
-	dPrngStateR = 0.0;
-}
-
-static inline int32_t random32(void)
-{
-	// LCG random 32-bit generator (quite good and fast)
-	randSeed *= 134775813;
-	randSeed++;
-	return randSeed;
-}
-
 #define NORM_FACTOR 2.0 /* nominally correct, but can clip from high-pass filter overshoot */
 
 static inline void processMixedSamplesAmigaPanning(int32_t i, int16_t *out)
 {
 	int32_t smp32;
-	double dPrng;
 
 	double dL = dMixBufferL[i];
 	double dR = dMixBufferR[i];
@@ -159,18 +140,12 @@ static inline void processMixedSamplesAmigaPanning(int32_t i, int16_t *out)
 	dL *= NORM_FACTOR * (-INT16_MAX / (double)PAULA_VOICES);
 	dR *= NORM_FACTOR * (-INT16_MAX / (double)PAULA_VOICES);
 
-	// left channel - 1-bit triangular dithering (high-pass filtered)
-	dPrng = random32() * (0.5 / INT32_MAX); // -0.5..0.5
-	dL = (dL + dPrng) - dPrngStateL;
-	dPrngStateL = dPrng;
+	// left channel
 	smp32 = (int32_t)dL;
 	CLAMP16(smp32);
 	out[0] = (int16_t)smp32;
 
-	// right channel - 1-bit triangular dithering (high-pass filtered)
-	dPrng = random32() * (0.5 / INT32_MAX); // -0.5..0.5
-	dR = (dR + dPrng) - dPrngStateR;
-	dPrngStateR = dPrng;
+	// right channel
 	smp32 = (int32_t)dR;
 	CLAMP16(smp32);
 	out[1] = (int16_t)smp32;
@@ -179,7 +154,6 @@ static inline void processMixedSamplesAmigaPanning(int32_t i, int16_t *out)
 static inline void processMixedSamples(int32_t i, int16_t *out)
 {
 	int32_t smp32;
-	double dPrng;
 
 	double dL = dMixBufferL[i];
 	double dR = dMixBufferR[i];
@@ -196,18 +170,12 @@ static inline void processMixedSamples(int32_t i, int16_t *out)
 	dL *= NORM_FACTOR * (-INT16_MAX / (double)PAULA_VOICES);
 	dR *= NORM_FACTOR * (-INT16_MAX / (double)PAULA_VOICES);
 
-	// left channel - 1-bit triangular dithering (high-pass filtered)
-	dPrng = random32() * (0.5 / INT32_MAX); // -0.5..0.5
-	dL = (dL + dPrng) - dPrngStateL;
-	dPrngStateL = dPrng;
+	// left channel
 	smp32 = (int32_t)dL;
 	CLAMP16(smp32);
 	out[0] = (int16_t)smp32;
 
-	// right channel - 1-bit triangular dithering (high-pass filtered)
-	dPrng = random32() * (0.5 / INT32_MAX); // -0.5..0.5
-	dR = (dR + dPrng) - dPrngStateR;
-	dPrngStateR = dPrng;
+	// right channel
 	smp32 = (int32_t)dR;
 	CLAMP16(smp32);
 	out[1] = (int16_t)smp32;
@@ -216,7 +184,7 @@ static inline void processMixedSamples(int32_t i, int16_t *out)
 static inline void processMixedSamplesAmigaPanning_2x(int32_t i, int16_t *out) // 2x oversampling
 {
 	int32_t smp32;
-	double dPrng, dL, dR;
+	double dL, dR;
 
 	// 2x downsampling (decimation)
 	const uint32_t offset1 = (i << 1) + 0;
@@ -228,18 +196,12 @@ static inline void processMixedSamplesAmigaPanning_2x(int32_t i, int16_t *out) /
 	dL *= NORM_FACTOR * (-INT16_MAX / (double)PAULA_VOICES);
 	dR *= NORM_FACTOR * (-INT16_MAX / (double)PAULA_VOICES);
 
-	// left channel - 1-bit triangular dithering (high-pass filtered)
-	dPrng = random32() * (0.5 / INT32_MAX); // -0.5..0.5
-	dL = (dL + dPrng) - dPrngStateL;
-	dPrngStateL = dPrng;
+	// left channel
 	smp32 = (int32_t)dL;
 	CLAMP16(smp32);
 	out[0] = (int16_t)smp32;
 
-	// right channel - 1-bit triangular dithering (high-pass filtered)
-	dPrng = random32() * (0.5 / INT32_MAX); // -0.5..0.5
-	dR = (dR + dPrng) - dPrngStateR;
-	dPrngStateR = dPrng;
+	// right channel
 	smp32 = (int32_t)dR;
 	CLAMP16(smp32);
 	out[1] = (int16_t)smp32;
@@ -248,7 +210,7 @@ static inline void processMixedSamplesAmigaPanning_2x(int32_t i, int16_t *out) /
 static inline void processMixedSamples_2x(int32_t i, int16_t *out) // 2x oversampling
 {
 	int32_t smp32;
-	double dPrng, dL, dR;
+	double dL, dR;
 
 	// 2x downsampling (decimation)
 	const uint32_t offset1 = (i << 1) + 0;
@@ -268,18 +230,12 @@ static inline void processMixedSamples_2x(int32_t i, int16_t *out) // 2x oversam
 	dL *= NORM_FACTOR * (-INT16_MAX / (double)PAULA_VOICES);
 	dR *= NORM_FACTOR * (-INT16_MAX / (double)PAULA_VOICES);
 
-	// left channel - 1-bit triangular dithering (high-pass filtered)
-	dPrng = random32() * (0.5 / INT32_MAX); // -0.5..0.5
-	dL = (dL + dPrng) - dPrngStateL;
-	dPrngStateL = dPrng;
+	// left channel
 	smp32 = (int32_t)dL;
 	CLAMP16(smp32);
 	out[0] = (int16_t)smp32;
 
-	// right channel - 1-bit triangular dithering (high-pass filtered)
-	dPrng = random32() * (0.5 / INT32_MAX); // -0.5..0.5
-	dR = (dR + dPrng) - dPrngStateR;
-	dPrngStateR = dPrng;
+	// right channel
 	smp32 = (int32_t)dR;
 	CLAMP16(smp32);
 	out[1] = (int16_t)smp32;
@@ -291,7 +247,7 @@ void outputAudio(int16_t *target, int32_t numSamples)
 	{
 		paulaGenerateSamples(dMixBufferL, dMixBufferR, numSamples*2);
 
-		// downsample, normalize and dither
+		// downsample and normalize
 		int16_t out[2];
 		int16_t *outStream = target;
 		if (stereoSeparation == 100)
@@ -317,7 +273,7 @@ void outputAudio(int16_t *target, int32_t numSamples)
 	{
 		paulaGenerateSamples(dMixBufferL, dMixBufferR, numSamples);
 
-		// normalize and dither
+		// normalize
 		int16_t out[2];
 		int16_t *outStream = target;
 		if (stereoSeparation == 100)
@@ -500,8 +456,14 @@ bool setupAudio(void)
 	audio.oversamplingFlag = (audio.outputRate < 96000); // we do 2x oversampling if the audio output rate is below 96kHz
 	audio.amigaModel = config.amigaModel;
 
+	uint32_t maxFrequency = audio.outputRate;
+	if (maxFrequency < config.mod2WavOutputFreq)
+		maxFrequency = config.mod2WavOutputFreq;
+
+	maxFrequency *= 2; // oversampling
+
 	const int32_t paulaMixFrequency = audio.oversamplingFlag ? audio.outputRate*2 : audio.outputRate;
-	int32_t maxSamplesPerTick = (int32_t)ceil(paulaMixFrequency / (MIN_BPM / 2.5)) + 1;
+	int32_t maxSamplesPerTick = (int32_t)ceil(maxFrequency / (MIN_BPM / 2.5)) + 1;
 
 	dMixBufferL = (double *)malloc(maxSamplesPerTick * sizeof (double));
 	dMixBufferR = (double *)malloc(maxSamplesPerTick * sizeof (double));
