@@ -355,7 +355,7 @@ static bool loadProTrackerDotIni(FILE *f)
 				config.autoFitVideoScale = true;
 				config.videoScaleFactor = 0; // will be set later
 			}
-			else if (lineLen >= 13 && configLine[12] == 'X' && isdigit(configLine[11]))
+			else if (lineLen >= 13 && toupper(configLine[12]) == 'X' && isdigit(configLine[11]))
 			{
 				config.autoFitVideoScale = false;
 				config.videoScaleFactor = (int8_t)(configLine[11] - '0');
@@ -430,7 +430,7 @@ static bool loadProTrackerDotIni(FILE *f)
 			if (configLine[17] != '\0')
 			{
 				const int32_t num = atoi(&configLine[17]);
-				config.mod2WavOutputFreq = CLAMP(num, 44100, 192000);
+				config.mod2WavOutputFreq = CLAMP(num, MIN_AUDIO_FREQUENCY, MAX_AUDIO_FREQUENCY);
 			}
 		}
 
@@ -440,7 +440,7 @@ static bool loadProTrackerDotIni(FILE *f)
 			if (configLine[10] != '\0')
 			{
 				const int32_t num = atoi(&configLine[10]);
-				config.soundFrequency = CLAMP(num, 44100, 192000);
+				config.soundFrequency = CLAMP(num, MIN_AUDIO_FREQUENCY, MAX_AUDIO_FREQUENCY);
 			}
 		}
 
