@@ -16,7 +16,9 @@ typedef struct config_t
 	bool waveformCenterLine, pattDots, compoMode, autoCloseDiskOp, hideDiskOpDates, hwMouse;
 	bool transDel, fullScreenStretch, vsyncOff, modDot, blankZeroFlag, realVuMeters, rememberPlayMode;
 	bool startInFullscreen, integerScaling, disableE8xEffect, noDownsampleOnSmpLoad, keepEditModeAfterStepPlay;
-	int8_t stereoSeparation, videoScaleFactor, accidental;
+	int8_t stereoSeparation, accidental;
+	bool autoFitVideoScale;
+	int8_t videoScaleFactor;
 	uint8_t pixelFilter, amigaModel;
 	uint16_t quantizeValue;
 	int32_t maxSampleLength;
