@@ -49,617 +49,6 @@ static const int8_t scancode2NoteHi[52] = // "USB usage page standard" order
 	-1, 24, 26, 28
 };
 
-// used for re-rendering text object while editing it
-void updateTextObject(int16_t editObject)
-{
-	switch (editObject)
-	{
-		default: break;
-		case PTB_SONGNAME: ui.updateSongName = true; break;
-		case PTB_SAMPLENAME: ui.updateCurrSampleName = true; break;
-		case PTB_PE_PATT: ui.updatePosEd = true; break;
-		case PTB_EO_QUANTIZE: ui.updateQuantizeText = true; break;
-		case PTB_EO_METRO_1: ui.updateMetro1Text = true; break;
-		case PTB_EO_METRO_2: ui.updateMetro2Text = true; break;
-		case PTB_EO_FROM_NUM: ui.updateFromText = true; break;
-		case PTB_EO_TO_NUM: ui.updateToText = true; break;
-		case PTB_EO_MIX: ui.updateMixText = true; break;
-		case PTB_EO_POS_NUM: ui.updatePosText = true; break;
-		case PTB_EO_MOD_NUM: ui.updateModText = true; break;
-		case PTB_EO_VOL_NUM: ui.updateVolText = true; break;
-		case PTB_DO_DATAPATH: ui.updateDiskOpPathText = true; break;
-		case PTB_POSS: ui.updateSongPos = true; break;
-		case PTB_PATTERNS: ui.updateSongPattern = true; break;
-		case PTB_LENGTHS: ui.updateSongLength = true; break;
-		case PTB_SAMPLES: ui.updateCurrSampleNum = true; break;
-		case PTB_SVOLUMES: ui.updateCurrSampleVolume = true; break;
-		case PTB_SLENGTHS: ui.updateCurrSampleLength = true; break;
-		case PTB_SREPEATS: ui.updateCurrSampleRepeat = true; break;
-		case PTB_SREPLENS: ui.updateCurrSampleReplen = true; break;
-		case PTB_PATTDATA: ui.updateCurrPattText = true; break;
-		case PTB_SA_VOL_FROM_NUM: ui.updateVolFromText = true; break;
-		case PTB_SA_VOL_TO_NUM: ui.updateVolToText = true; break;
-		case PTB_SA_FIL_LP_CUTOFF: ui.updateLPText = true; break;
-		case PTB_SA_FIL_HP_CUTOFF: ui.updateHPText = true; break;
-	}
-}
-
-void exitGetTextLine(bool updateValue)
-{
-	int8_t tmp8;
-	int16_t tmp16;
-	int32_t tmp32;
-
-	SDL_StopTextInput();
-
-	// if user updated the disk op path text
-	if (ui.diskOpScreenShown && ui.editObject == PTB_DO_DATAPATH)
-	{
-		UNICHAR *pathU = (UNICHAR *)calloc(PATH_MAX + 2, sizeof (UNICHAR));
-		if (pathU != NULL)
-		{
-#ifdef _WIN32
-			MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, editor.currPath, -1, pathU, PATH_MAX);
-#else
-			strcpy(pathU, editor.currPath);
-#endif
-			diskOpSetPath(pathU, DISKOP_CACHE);
-			free(pathU);
-		}
-	}
-
-	if (ui.editTextType != TEXT_EDIT_STRING)
-	{
-		if (ui.dstPos != ui.numLen)
-			removeTextEditMarker();
-
-		updateTextObject(ui.editObject);
-	}
-	else
-	{
-		removeTextEditMarker();
-
-		// yet another kludge...
-		if (ui.editObject == PTB_PE_PATT)
-			ui.updatePosEd = true;
-	}
-
-	ui.editTextFlag = false;
-
-	ui.lineCurX = 0;
-	ui.lineCurY = 0;
-	ui.editPos = NULL;
-	ui.dstPos = 0;
-
-	if (ui.editTextType == TEXT_EDIT_STRING)
-	{
-		if (ui.dstOffset != NULL)
-			*ui.dstOffset = '\0';
-
-		pointerSetPreviousMode();
-
-		if (!editor.mixFlag)
-			updateWindowTitle(MOD_IS_MODIFIED);
-	}
-	else
-	{
-		// set back GUI text pointers and update values (if requested)
-
-		moduleSample_t *s = &song->samples[editor.currSample];
-		switch (ui.editObject)
-		{
-			case PTB_SA_FIL_LP_CUTOFF:
-			{
-				editor.lpCutOffDisp = &editor.lpCutOff;
-
-				if (updateValue)
-				{
-					editor.lpCutOff = ui.tmpDisp16;
-					if (editor.lpCutOff > (uint16_t)(FILTERS_BASE_FREQ/2))
-						editor.lpCutOff = (uint16_t)(FILTERS_BASE_FREQ/2);
-
-					ui.updateLPText = true;
-				}
-			}
-			break;
-
-			case PTB_SA_FIL_HP_CUTOFF:
-			{
-				editor.hpCutOffDisp = &editor.hpCutOff;
-
-				if (updateValue)
-				{
-					editor.hpCutOff = ui.tmpDisp16;
-					if (editor.hpCutOff > (uint16_t)(FILTERS_BASE_FREQ/2))
-						editor.hpCutOff = (uint16_t)(FILTERS_BASE_FREQ/2);
-
-					ui.updateHPText = true;
-				}
-			}
-			break;
-
-			case PTB_SA_VOL_FROM_NUM:
-			{
-				editor.vol1Disp = &editor.vol1;
-
-				if (updateValue)
-				{
-					editor.vol1 = ui.tmpDisp16;
-					if (editor.vol1 > 200)
-						editor.vol1 = 200;
-
-					ui.updateVolFromText = true;
-					showVolFromSlider();
-				}
-			}
-			break;
-
-			case PTB_SA_VOL_TO_NUM:
-			{
-				editor.vol2Disp = &editor.vol2;
-
-				if (updateValue)
-				{
-					editor.vol2 = ui.tmpDisp16;
-					if (editor.vol2 > 200)
-						editor.vol2 = 200;
-
-					ui.updateVolToText = true;
-					showVolToSlider();
-				}
-			}
-			break;
-
-			case PTB_EO_VOL_NUM:
-			{
-				editor.sampleVolDisp = &editor.sampleVol;
-
-				if (updateValue)
-				{
-					editor.sampleVol = ui.tmpDisp16;
-					ui.updateVolText = true;
-				}
-			}
-			break;
-
-			case PTB_EO_POS_NUM:
-			{
-				editor.samplePosDisp = &editor.samplePos;
-
-				if (updateValue)
-				{
-					editor.samplePos = ui.tmpDisp32;
-					if (editor.samplePos > config.maxSampleLength)
-						editor.samplePos = config.maxSampleLength;
-
-					if (editor.samplePos > song->samples[editor.currSample].length)
-						editor.samplePos = song->samples[editor.currSample].length;
-
-					ui.updatePosText = true;
-				}
-			}
-			break;
-
-			case PTB_EO_QUANTIZE:
-			{
-				editor.quantizeValueDisp = &config.quantizeValue;
-
-				if (updateValue)
-				{
-					if (ui.tmpDisp16 > 63)
-						ui.tmpDisp16 = 63;
-
-					config.quantizeValue = ui.tmpDisp16;
-					ui.updateQuantizeText = true;
-				}
-			}
-			break;
-
-			case PTB_EO_METRO_1: // metronome speed
-			{
-				editor.metroSpeedDisp = &editor.metroSpeed;
-
-				if (updateValue)
-				{
-					if (ui.tmpDisp16 > 64)
-						ui.tmpDisp16 = 64;
-
-					editor.metroSpeed = ui.tmpDisp16;
-					ui.updateMetro1Text = true;
-				}
-			}
-			break;
-
-			case PTB_EO_METRO_2: // metronome channel
-			{
-				editor.metroChannelDisp = &editor.metroChannel;
-
-				if (updateValue)
-				{
-					if (ui.tmpDisp16 > 4)
-						ui.tmpDisp16 = 4;
-
-					editor.metroChannel = ui.tmpDisp16;
-					ui.updateMetro2Text = true;
-				}
-			}
-			break;
-
-			case PTB_EO_FROM_NUM:
-			{
-				editor.sampleFromDisp = &editor.sampleFrom;
-
-				if (updateValue)
-				{
-					editor.sampleFrom = ui.tmpDisp8;
-
-					// signed check + normal check
-					if (editor.sampleFrom < 0x00 || editor.sampleFrom > 0x1F)
-						editor.sampleFrom = 0x1F;
-
-					ui.updateFromText = true;
-				}
-			}
-			break;
-
-			case PTB_EO_TO_NUM:
-			{
-				editor.sampleToDisp = &editor.sampleTo;
-
-				if (updateValue)
-				{
-					editor.sampleTo = ui.tmpDisp8;
-
-					// signed check + normal check
-					if (editor.sampleTo < 0x00 || editor.sampleTo > 0x1F)
-						editor.sampleTo = 0x1F;
-
-					ui.updateToText = true;
-				}
-			}
-			break;
-
-			case PTB_PE_PATT:
-			{
-				int16_t posEdPos = song->currOrder;
-				if (posEdPos > song->header.numOrders-1)
-					posEdPos = song->header.numOrders-1;
-
-				editor.currPosEdPattDisp = &song->header.order[posEdPos];
-
-				if (updateValue)
-				{
-					if (ui.tmpDisp16 > MAX_PATTERNS-1)
-						ui.tmpDisp16 = MAX_PATTERNS-1;
-
-					song->header.order[posEdPos] = ui.tmpDisp16;
-
-					updateWindowTitle(MOD_IS_MODIFIED);
-
-					if (ui.posEdScreenShown)
-						ui.updatePosEd = true;
-
-					ui.updateSongPattern = true;
-					ui.updateSongSize = true;
-				}
-			}
-			break;
-
-			case PTB_POSS:
-			{
-				editor.currPosDisp = &song->currOrder;
-
-				if (updateValue)
-				{
-					tmp16 = ui.tmpDisp16;
-					if (tmp16 > 126)
-						tmp16 = 126;
-
-					if (song->currOrder != tmp16)
-					{
-						song->currOrder = tmp16;
-						editor.currPatternDisp = &song->header.order[song->currOrder];
-
-						if (ui.posEdScreenShown)
-							ui.updatePosEd = true;
-
-						ui.updateSongPos = true;
-						ui.updatePatternData = true;
-					}
-				}
-			}
-			break;
-
-			case PTB_PATTERNS:
-			{
-				editor.currPatternDisp = &song->header.order[song->currOrder];
-
-				if (updateValue)
-				{
-					tmp16 = ui.tmpDisp16;
-					if (tmp16 > MAX_PATTERNS-1)
-						tmp16 = MAX_PATTERNS-1;
-
-					if (song->header.order[song->currOrder] != tmp16)
-					{
-						song->header.order[song->currOrder] = tmp16;
-
-						updateWindowTitle(MOD_IS_MODIFIED);
-
-						if (ui.posEdScreenShown)
-							ui.updatePosEd = true;
-
-						ui.updateSongPattern = true;
-						ui.updateSongSize = true;
-					}
-				}
-			}
-			break;
-
-			case PTB_LENGTHS:
-			{
-				editor.currLengthDisp = &song->header.numOrders;
-
-				if (updateValue)
-				{
-					tmp16 = CLAMP(ui.tmpDisp16, 1, 127);
-
-					if (song->header.numOrders != tmp16)
-					{
-						song->header.numOrders = tmp16;
-
-						int16_t posEdPos = song->currOrder;
-						if (posEdPos > song->header.numOrders-1)
-							posEdPos = song->header.numOrders-1;
-
-						editor.currPosEdPattDisp = &song->header.order[posEdPos];
-
-						if (ui.posEdScreenShown)
-							ui.updatePosEd = true;
-
-						ui.updateSongLength = true;
-						ui.updateSongSize = true;
-						updateWindowTitle(MOD_IS_MODIFIED);
-					}
-				}
-			}
-			break;
-
-			case PTB_PATTDATA:
-			{
-				editor.currEditPatternDisp = &song->currPattern;
-
-				if (updateValue)
-				{
-					if (song->currPattern != ui.tmpDisp16)
-					{
-						setPattern(ui.tmpDisp16);
-						ui.updatePatternData = true;
-						ui.updateCurrPattText = true;
-					}
-				}
-			}
-			break;
-
-			case PTB_SAMPLES:
-			{
-				editor.currSampleDisp = &editor.currSample;
-
-				if (updateValue)
-				{
-					tmp8 = ui.tmpDisp8;
-					if (tmp8 < 0x00) // (signed) if >0x7F was entered, clamp to 0x1F
-						tmp8 = 0x1F;
-
-					tmp8 = CLAMP(tmp8, 0x01, 0x1F) - 1;
-
-					if (tmp8 != editor.currSample)
-					{
-						editor.currSample = tmp8;
-						updateCurrSample();
-					}
-				}
-			}
-			break;
-
-			case PTB_SVOLUMES:
-			{
-				s->volumeDisp = &s->volume;
-
-				if (updateValue)
-				{
-					tmp8 = ui.tmpDisp8;
-
-					// signed check + normal check
-					if (tmp8 < 0x00 || tmp8 > 0x40)
-						 tmp8 = 0x40;
-
-					if (s->volume != tmp8)
-					{
-						s->volume = tmp8;
-						ui.updateCurrSampleVolume = true;
-						updateWindowTitle(MOD_IS_MODIFIED);
-					}
-				}
-			}
-			break;
-
-			case PTB_SLENGTHS:
-			{
-				s->lengthDisp = &s->length;
-
-				if (updateValue)
-				{
-					tmp32 = ui.tmpDisp32 & ~1; // even'ify
-					if (tmp32 > config.maxSampleLength)
-						tmp32 = config.maxSampleLength;
-
-					if (s->loopStart+s->loopLength > 2)
-					{
-						if (tmp32 < s->loopStart+s->loopLength)
-							tmp32 = s->loopStart+s->loopLength;
-					}
-
-					tmp32 &= ~1;
-
-					if (s->length != tmp32)
-					{
-						turnOffVoices();
-						s->length = tmp32;
-
-						ui.updateCurrSampleLength = true;
-						ui.updateSongSize = true;
-						updateSamplePos();
-
-						if (ui.samplerScreenShown)
-							redrawSample();
-
-						recalcChordLength();
-						updateWindowTitle(MOD_IS_MODIFIED);
-					}
-				}
-			}
-			break;
-
-			case PTB_SREPEATS:
-			{
-				s->loopStartDisp = &s->loopStart;
-
-				if (updateValue)
-				{
-					tmp32 = ui.tmpDisp32 & ~1; // even'ify
-					if (tmp32 > config.maxSampleLength)
-						tmp32 = config.maxSampleLength;
-
-					if (s->length >= s->loopLength)
-					{
-						if (tmp32+s->loopLength > s->length)
-							 tmp32 = s->length - s->loopLength;
-					}
-					else
-					{
-						tmp32 = 0;
-					}
-
-					tmp32 &= ~1;
-
-					if (s->loopStart != tmp32)
-					{
-						turnOffVoices();
-						s->loopStart = tmp32;
-						updatePaulaLoops();
-
-						ui.updateCurrSampleRepeat = true;
-
-						if (ui.editOpScreenShown && ui.editOpScreen == 3)
-							ui.updateChordLengthText = true;
-
-						if (ui.samplerScreenShown)
-							setLoopSprites();
-
-						updateWindowTitle(MOD_IS_MODIFIED);
-					}
-				}
-			}
-			break;
-
-			case PTB_SREPLENS:
-			{
-				s->loopLengthDisp = &s->loopLength;
-
-				if (updateValue)
-				{
-					tmp32 = ui.tmpDisp32 & ~1; // even'ify
-					if (tmp32 > config.maxSampleLength)
-						tmp32 = config.maxSampleLength;
-
-					if (s->length >= s->loopStart)
-					{
-						if (s->loopStart+tmp32 > s->length)
-							tmp32 = s->length - s->loopStart;
-					}
-					else
-					{
-						tmp32 = 2;
-					}
-
-					tmp32 &= ~1;
-
-					if (tmp32 < 2)
-						tmp32 = 2;
-
-					if (s->loopLength != tmp32)
-					{
-						turnOffVoices();
-						s->loopLength = tmp32;
-						updatePaulaLoops();
-
-						ui.updateCurrSampleReplen = true;
-						if (ui.editOpScreenShown && ui.editOpScreen == 3)
-							ui.updateChordLengthText = true;
-
-						if (ui.samplerScreenShown)
-							setLoopSprites();
-
-						updateWindowTitle(MOD_IS_MODIFIED);
-					}
-				}
-			}
-			break;
-
-			default: break;
-		}
-
-		pointerSetPreviousMode();
-	}
-
-	ui.editTextType = 0;
-}
-
-void getTextLine(int16_t editObject)
-{
-	pointerSetMode(POINTER_MODE_MSG1, NO_CARRY);
-
-	ui.lineCurY = (ui.editTextPos / 40) + 5;
-	ui.lineCurX = ((ui.editTextPos % 40) * FONT_CHAR_W) + 4;
-	ui.dstPtr = ui.showTextPtr;
-	ui.editPos = ui.showTextPtr;
-	ui.dstPos = 0;
-	ui.editTextFlag = true;
-	ui.editTextType = TEXT_EDIT_STRING;
-	ui.editObject = editObject;
-
-	if (ui.dstOffset != NULL)
-		ui.dstOffset[0] = '\0';
-
-	// kludge
-	if (editor.mixFlag)
-	{
-		textCharNext();
-		textCharNext();
-		textCharNext();
-		textCharNext();
-	}
-
-	renderTextEditMarker();
-	SDL_StartTextInput();
-}
-
-void getNumLine(uint8_t type, int16_t editObject)
-{
-	pointerSetMode(POINTER_MODE_MSG1, NO_CARRY);
-
-	ui.lineCurY = (ui.editTextPos / 40) + 5;
-	ui.lineCurX = ((ui.editTextPos % 40) * FONT_CHAR_W) + 4;
-	ui.dstPos = 0;
-	ui.editTextFlag = true;
-	ui.editTextType = type;
-	ui.editObject = editObject;
-
-	renderTextEditMarker();
-	SDL_StartTextInput();
-}
-
 static uint8_t quantizeCheck(uint8_t row)
 {
 	assert(song != NULL);
