@@ -35,10 +35,12 @@
 #include "pt2_sampling.h"
 #include "pt2_askbox.h"
 #include "pt2_replayer.h"
+#include "pt2_textedit.h"
 
-#define CRASH_TEXT "Oh no!\nThe ProTracker 2 clone has crashed...\n\nA backup .mod was hopefully " \
+#define CRASH_TEXT "Oh no! The ProTracker 2 clone has crashed...\nA backup .mod was hopefully " \
                    "saved to the current module directory.\n\nPlease report this bug if you can.\n" \
-                   "Try to mention what you did before the crash happened."
+                   "Try to mention what you did before the crash happened.\n" \
+                   "My email is on the bottom of https://16-bits.org"
 
 module_t *song = NULL; // globalized
 
@@ -88,15 +90,16 @@ static void cleanUp(void);
 
 static void clearStructs(void)
 {
-	memset(&keyb,   0, sizeof (keyb));
-	memset(&mouse,  0, sizeof (mouse));
-	memset(&video,  0, sizeof (video));
+	memset(&keyb, 0, sizeof (keyb));
+	memset(&mouse, 0, sizeof (mouse));
+	memset(&video, 0, sizeof (video));
 	memset(&editor, 0, sizeof (editor));
 	memset(&diskop, 0, sizeof (diskop));
 	memset(&cursor, 0, sizeof (cursor));
-	memset(&ui,     0, sizeof (ui));
+	memset(&ui, 0, sizeof (ui));
 	memset(&config, 0, sizeof (config));
-	memset(&audio,  0, sizeof (audio));
+	memset(&audio, 0, sizeof (audio));
+	memset(&textEdit, 0, sizeof (textEdit));
 
 	audio.rescanAudioDevicesSupported = true;
 }
@@ -165,13 +168,12 @@ int main(int argc, char *argv[])
 	}
 #endif
 
-#ifdef _WIN32
-
-	// ALT+F4 is used in ProTracker, but is "close program" in Windows...
+	// ALT+F4 is used in ProTracker, but is "close program" in some cases...
 #if SDL_MINOR_VERSION >= 24 || (SDL_MINOR_VERSION == 0 && SDL_PATCHLEVEL >= 4)
-	SDL_SetHint(SDL_HINT_WINDOWS_NO_CLOSE_ON_ALT_F4, "1");
+	SDL_SetHint("SDL_WINDOWS_NO_CLOSE_ON_ALT_F4", "1");
 #endif
 
+#ifdef _WIN32
 #ifndef _MSC_VER
 	SetProcessDPIAware();
 #endif
@@ -247,7 +249,7 @@ int main(int argc, char *argv[])
 
 	loadConfig();
 
-	if (!setupVideo())
+	if (!allocSamplerVars() || !setupVideo())
 	{
 		cleanUp();
 		SDL_Quit();
@@ -288,10 +290,10 @@ int main(int argc, char *argv[])
 
 	// setup GUI text pointers (for recently allocated song structure)
 	editor.currEditPatternDisp = &song->currPattern;
-	editor.currPosDisp = &song->currOrder;
-	editor.currPatternDisp = &song->header.order[0];
-	editor.currPosEdPattDisp = &song->header.order[0];
-	editor.currLengthDisp = &song->header.numOrders;
+	editor.currPosDisp = &song->currPos;
+	editor.currPatternDisp = &song->header.patternTable[0];
+	editor.currPosEdPattDisp = &song->header.patternTable[0];
+	editor.currLengthDisp = &song->header.songLength;
 
 	if (!initScopes())
 	{
@@ -372,7 +374,6 @@ int main(int argc, char *argv[])
 
 static void handleInput(void)
 {
-	
 	SDL_Event event;
 	while (SDL_PollEvent(&event))
 	{
@@ -389,7 +390,8 @@ static void handleInput(void)
 		}
 
 #ifdef _WIN32
-		handleSysMsg(event);
+		if (event.type == SDL_SYSWMEVENT)
+			handleSysMsg(event);
 #endif
 		if (ui.editTextFlag && event.type == SDL_TEXTINPUT)
 		{
@@ -409,11 +411,16 @@ static void handleInput(void)
 		}
 		else if (event.type == SDL_DROPFILE)
 		{
+			if (!video.fullscreen)
+			{
+				if (SDL_GetWindowFlags(video.window) & SDL_WINDOW_MINIMIZED)
+					SDL_RestoreWindow(video.window);
+
+				SDL_RaiseWindow(video.window);
+			}
+
 			loadDroppedFile(event.drop.file, (uint32_t)strlen(event.drop.file), false, true);
 			SDL_free(event.drop.file);
-
-			SDL_RestoreWindow(video.window);
-			SDL_RaiseWindow(video.window);
 		}
 		else if (event.type == SDL_QUIT)
 		{
@@ -494,14 +501,13 @@ static bool initializeVars(void)
 
 	// allocate some memory
 
-	if (!allocSamplerVars() || !allocDiskOpVars())
+	if (!allocDiskOpVars())
 		goto oom;
 
 	config.defModulesDir = (char *)calloc(PATH_MAX + 1, sizeof (char));
 	config.defSamplesDir = (char *)calloc(PATH_MAX + 1, sizeof (char));
-	editor.tempSample = (int8_t *)calloc(131070, 1);
 
-	if (config.defModulesDir == NULL || config.defSamplesDir == NULL || editor.tempSample == NULL)
+	if (config.defModulesDir == NULL || config.defSamplesDir == NULL)
 		goto oom;
 
 	turnOffVoices();
@@ -594,7 +600,9 @@ static void handleSigTerm(void)
 		if (!video.fullscreen)
 		{
 			// de-minimize window and set focus so that the user sees the message box
-			SDL_RestoreWindow(video.window);
+			if (SDL_GetWindowFlags(video.window) & SDL_WINDOW_MINIMIZED)
+				SDL_RestoreWindow(video.window);
+
 			SDL_RaiseWindow(video.window);
 		}
 
@@ -838,9 +846,6 @@ static bool handleSingleInstancing(int32_t argc, char **argv)
 
 static void handleSysMsg(SDL_Event inputEvent)
 {
-	if (inputEvent.type != SDL_SYSWMEVENT)
-		return;
-
 	SDL_SysWMmsg *wmMsg = inputEvent.syswm.msg;
 	if (wmMsg->subsystem == SDL_SYSWM_WINDOWS && wmMsg->msg.win.msg == SYSMSG_FILE_ARG)
 	{
@@ -850,16 +855,18 @@ static void handleSysMsg(SDL_Event inputEvent)
 			sharedMemBuf = (LPTSTR)MapViewOfFile(hMapFile, FILE_MAP_READ, 0, 0, ARGV_SHARED_MEM_MAX_LEN);
 			if (sharedMemBuf != NULL)
 			{
+				if (video.window != NULL && !video.fullscreen)
+				{
+					if (SDL_GetWindowFlags(video.window) & SDL_WINDOW_MINIMIZED)
+						SDL_RestoreWindow(video.window);
+
+					SDL_RaiseWindow(video.window);
+				}
+
 				loadDroppedFile((char *)sharedMemBuf, (uint32_t)strlen(sharedMemBuf), true, true);
 
 				UnmapViewOfFile(sharedMemBuf);
 				sharedMemBuf = NULL;
-
-				if (video.window != NULL)
-				{
-					SDL_RestoreWindow(video.window);
-					SDL_RaiseWindow(video.window);
-				}
 			}
 
 			CloseHandle(hMapFile);
@@ -935,7 +942,6 @@ static void cleanUp(void) // never call this inside the main loop!
 
 	if (config.defModulesDir != NULL) free(config.defModulesDir);
 	if (config.defSamplesDir != NULL) free(config.defSamplesDir);
-	if (editor.tempSample != NULL) free(editor.tempSample);
 
 #ifdef _WIN32
 #ifndef _DEBUG
