@@ -45,9 +45,7 @@ static paulaVoice_t paula[PAULA_VOICES];
 
 void paulaSetup(double dOutputFreq, uint32_t amigaModel)
 {
-	if (dOutputFreq <= 0.0)
-		dOutputFreq = 44100.0;
-
+	assert(dOutputFreq != 0.0);
 	dPaulaOutputFreq = dOutputFreq;
 	dPeriodToDeltaDiv = PAULA_PAL_CLK / dPaulaOutputFreq;
 
@@ -110,7 +108,7 @@ void paulaSetup(double dOutputFreq, uint32_t amigaModel)
 
 	if (amigaModel == MODEL_A500)
 	{
-		// A500 1-pole (6db/oct) RC low-pass filter:
+		// A500 1-pole (6dB/oct) RC low-pass filter:
 		R = 360.0; // R321 (360 ohm)
 		C = 1e-7;  // C321 (0.1uF)
 		cutoff = 1.0 / (PT2_TWO_PI * R * C); // ~4420.971Hz
@@ -137,6 +135,8 @@ void paulaSetup(double dOutputFreq, uint32_t amigaModel)
 		cutoff = 1.0 / (PT2_TWO_PI * R * C); // ~5.319Hz
 		setupOnePoleFilter(dPaulaOutputFreq, cutoff, &filterHi);
 	}
+
+	// Note: A500 rev3 (old) -may- be C1 = 7500pF (cutoff = 2942.776Hz, qfactor = 0.693375)
 	
 	// 2-pole (12dB/oct) RC low-pass filter ("LED" filter, same values on A500/A1200):
 	R1 = 10000.0; // R322 (10K ohm)
