@@ -15,12 +15,13 @@
 #include "pt2_scopes.h"
 #include "pt2_paula.h"
 #include "pt2_visuals_sync.h"
+#include "pt2_posed.h"
 
 static bool posJumpAssert, pBreakFlag, modRenderDone;
 static bool doStopSong; // from F00 (Set Speed)
 static int8_t pBreakPosition, oldRow, modPattern;
 static uint8_t pattDelTime, lowMask = 0xFF, pattDelTime2;
-static int16_t modOrder, oldPattern, oldOrder;
+static int16_t modPos, oldPattern, oldPos;
 static uint16_t DMACONtemp;
 static int32_t modBPM, oldBPM, oldSpeed, ciaSetBPM;
 
@@ -30,6 +31,13 @@ static const uint8_t funkTable[16] = // EFx (FunkRepeat/InvertLoop)
 	0x10, 0x13, 0x16, 0x1A, 0x20, 0x2B, 0x40, 0x80
 };
 
+void gotoNextMulti(void)
+{
+	cursor.channel = (editor.multiModeNext[cursor.channel] - 1) & 3;
+	cursor.pos = cursor.channel * 6;
+	updateCursorPos();
+}
+
 double ciaBpm2Hz(int32_t bpm)
 {
 	if (bpm == 0)
@@ -53,6 +61,8 @@ void updatePaulaLoops(void) // used after manipulating sample loop points while
 			const moduleSample_t *s = &song->samples[editor.currSample];
 
 			const uint32_t voiceAddr = 0xDFF0A0 + (i * 16);
+
+			// set voice data ptr and data length
 			paulaWritePtr(voiceAddr + 0, ch->n_start + s->loopStart);
 			paulaWriteWord(voiceAddr + 4, (uint16_t)(s->loopLength >> 1));
 
@@ -83,9 +93,6 @@ void turnOffVoices(void)
 		setVisualsVolume(i, 0);
 	}
 
-	// reset dithering/filter states (so that every playback session is identical)
-	resetAudioDithering();
-
 	if (audioWasntLocked)
 		unlockAudio();
 
@@ -112,7 +119,7 @@ void setReplayerPosToTrackerPos(void)
 		return;
 
 	modPattern = (int8_t)song->currPattern;
-	modOrder = song->currOrder;
+	modPos = song->currPos;
 	song->row = song->currRow;
 	song->tick = 0;
 }
@@ -137,7 +144,7 @@ module_t *createEmptyMod(void)
 			goto error;
 	}
 
-	m->header.numOrders = 1;
+	m->header.songLength = 1;
 
 	moduleSample_t *s = m->samples;
 	for (int32_t i = 0; i < MOD_SAMPLES; i++, s++)
@@ -222,7 +229,7 @@ void storeTempVariables(void) // this one is accessed in other files, so non-sta
 {
 	oldBPM = song->currBPM;
 	oldRow = song->currRow;
-	oldOrder = song->currOrder;
+	oldPos = song->currPos;
 	oldSpeed = song->currSpeed;
 	oldPattern = song->currPattern;
 }
@@ -309,7 +316,7 @@ static void jumpLoop(moduleChannel_t *ch)
 		if (editor.mod2WavOngoing)
 		{
 			for (int32_t tempParam = pBreakPosition; tempParam <= song->row; tempParam++)
-				editor.rowVisitTable[(modOrder * MOD_ROWS) + tempParam] = false;
+				editor.rowVisitTable[(modPos * MOD_ROWS) + tempParam] = false;
 		}
 	}
 }
@@ -353,24 +360,27 @@ static void doRetrg(moduleChannel_t *ch)
 {
 	const uint32_t voiceAddr = 0xDFF0A0 + (ch->n_chanindex * 16);
 
-	paulaWriteWord(0xDFF096, ch->n_dmabit); // voice DMA off
+	// voice DMA off
+	paulaWriteWord(0xDFF096, ch->n_dmabit); 
+
+	// set voice data ptr, data length and period
 	paulaWritePtr(voiceAddr + 0, ch->n_start); // n_start is increased on 9xx
 	paulaWriteWord(voiceAddr + 4, ch->n_length);
 	paulaWriteWord(voiceAddr + 6, ch->n_period);
-	paulaWriteWord(0xDFF096, 0x8000 | ch->n_dmabit); // voice DMA on
+
+	// voice DMA on
+	paulaWriteWord(0xDFF096, 0x8000 | ch->n_dmabit); 
 	
-	// these take effect after the current DMA cycle is done
+	// set new data ptr and data length (these take effect after the current DMA cycle is done)
 	paulaWritePtr(voiceAddr + 0, ch->n_loopstart);
 	paulaWriteWord(voiceAddr + 4, ch->n_replen);
 
 	// update tracker visuals
-
 	setVisualsDMACON(ch->n_dmabit);
 	setVisualsDataPtr(ch->n_chanindex, ch->n_start);
 	setVisualsLength(ch->n_chanindex, ch->n_length);
 	setVisualsPeriod(ch->n_chanindex, ch->n_period);
 	setVisualsDMACON(0x8000 | ch->n_dmabit);
-
 	setVisualsDataPtr(ch->n_chanindex, ch->n_loopstart);
 	setVisualsLength(ch->n_chanindex, ch->n_replen);
 
@@ -464,7 +474,7 @@ static void positionJump(moduleChannel_t *ch)
 {
 	// original PT doesn't do this check, but we have to
 	if (editor.playMode != PLAY_MODE_PATTERN || (editor.currMode == MODE_RECORD && editor.recordMode != RECORD_PATT))
-		modOrder = (ch->n_cmd & 0xFF) - 1; // B00 results in -1, but it safely jumps to order 0
+		modPos = (ch->n_cmd & 0xFF) - 1; // B00 results in -1, but it safely jumps to order 0
 
 	pBreakPosition = 0;
 	posJumpAssert = true;
@@ -519,8 +529,11 @@ static void arpeggio(moduleChannel_t *ch)
 	}
 	else // arpTick 0
 	{
+		// set voice period
 		paulaWriteWord(voiceAddr + 6, ch->n_period);
+
 		setVisualsPeriod(ch->n_chanindex, ch->n_period);
+
 		return;
 	}
 
@@ -534,8 +547,11 @@ static void arpeggio(moduleChannel_t *ch)
 	{
 		if (ch->n_period >= periods[baseNote])
 		{
+			// set voice period
 			paulaWriteWord(voiceAddr + 6, periods[baseNote+arpNote]);
+
 			setVisualsPeriod(ch->n_chanindex, periods[baseNote+arpNote]);
+
 			break;
 		}
 	}
@@ -549,6 +565,7 @@ static void portaUp(moduleChannel_t *ch)
 	if ((ch->n_period & 0xFFF) < 113) // PT BUG: sign removed before comparison, underflow not clamped!
 		ch->n_period = (ch->n_period & 0xF000) | 113;
 
+	// set voice period
 	const uint32_t voiceAddr = 0xDFF0A0 + (ch->n_chanindex * 16);
 	paulaWriteWord(voiceAddr + 6, ch->n_period & 0xFFF);
 
@@ -563,6 +580,7 @@ static void portaDown(moduleChannel_t *ch)
 	if ((ch->n_period & 0xFFF) > 856)
 		ch->n_period = (ch->n_period & 0xF000) | 856;
 
+	// set voice period
 	const uint32_t voiceAddr = 0xDFF0A0 + (ch->n_chanindex * 16);
 	paulaWriteWord(voiceAddr + 6, ch->n_period & 0xFFF);
 
@@ -574,6 +592,8 @@ static void filterOnOff(moduleChannel_t *ch)
 	if (song->tick == 0) // added this (just pointless to call this during all ticks!)
 	{
 		const bool filterOn = (ch->n_cmd & 1) ^ 1;
+
+		// set "LED" filter
 		paulaWriteByte(0xBFE001, filterOn << 1);
 		audio.ledFilterEnabled = filterOn;
 	}
@@ -654,7 +674,9 @@ static void tonePortNoChange(moduleChannel_t *ch)
 
 	if ((ch->n_glissfunk & 0xF) == 0)
 	{
+		// set voice period
 		paulaWriteWord(voiceAddr + 6, ch->n_period);
+
 		setVisualsPeriod(ch->n_chanindex, ch->n_period);
 	}
 	else
@@ -675,7 +697,9 @@ static void tonePortNoChange(moduleChannel_t *ch)
 			}
 		}
 
+		// set voice period
 		paulaWriteWord(voiceAddr + 6, portaPointer[i]);
+
 		setVisualsPeriod(ch->n_chanindex, portaPointer[i]);
 	}
 }
@@ -721,8 +745,9 @@ static void vibrato2(moduleChannel_t *ch)
 	else
 		vibratoData = ch->n_period - vibratoData;
 
+	// set voice period
 	const uint32_t voiceAddr = 0xDFF0A0 + (ch->n_chanindex * 16);
-	paulaWriteWord(voiceAddr + 6, vibratoData); // period
+	paulaWriteWord(voiceAddr + 6, vibratoData);
 
 	setVisualsPeriod(ch->n_chanindex, vibratoData);
 
@@ -796,8 +821,9 @@ static void tremolo(moduleChannel_t *ch)
 			tremoloData = 0;
 	}
 
+	// set voice volume
 	const uint32_t voiceAddr = 0xDFF0A0 + (ch->n_chanindex * 16);
-	paulaWriteWord(voiceAddr + 8, tremoloData); // volume
+	paulaWriteWord(voiceAddr + 8, tremoloData);
 
 	setVisualsVolume(ch->n_chanindex, tremoloData);
 
@@ -878,6 +904,7 @@ static void checkMoreEffects(moduleChannel_t *ch)
 		return;
 	}
 
+	// set voice period
 	const uint32_t voiceAddr = 0xDFF0A0 + (ch->n_chanindex * 16);
 	paulaWriteWord(voiceAddr + 6, ch->n_period);
 
@@ -905,6 +932,7 @@ static void chkefx2(moduleChannel_t *ch)
 		default: break;
 	}
 
+	// set voice period
 	const uint32_t voiceAddr = 0xDFF0A0 + (ch->n_chanindex * 16);
 	paulaWriteWord(voiceAddr + 6, ch->n_period);
 
@@ -933,6 +961,7 @@ static void checkEffects(moduleChannel_t *ch)
 	const uint8_t cmd = (ch->n_cmd & 0x0F00) >> 8;
 	if (cmd != 0x7)
 	{
+		// set voice volume
 		const uint32_t voiceAddr = 0xDFF0A0 + (ch->n_chanindex * 16);
 		paulaWriteWord(voiceAddr + 8, ch->n_volume);
 
@@ -957,13 +986,15 @@ static void setPeriod(moduleChannel_t *ch)
 
 	if ((ch->n_cmd & 0xFF0) != 0xED0) // no note delay
 	{
-		paulaWriteWord(0xDFF096, ch->n_dmabit); // voice DMA off (turned on in setDMA() later)
+		// voice DMA off (turned on in setDMA() later)
+		paulaWriteWord(0xDFF096, ch->n_dmabit);
 
 		if ((ch->n_wavecontrol & 0x04) == 0) ch->n_vibratopos = 0;
 		if ((ch->n_wavecontrol & 0x40) == 0) ch->n_tremolopos = 0;
 
 		const uint32_t voiceAddr = 0xDFF0A0 + (ch->n_chanindex * 16);
 
+		// set voice data length and data ptr
 		paulaWriteWord(voiceAddr + 4, ch->n_length);
 		paulaWritePtr(voiceAddr + 0, ch->n_start);
 
@@ -974,6 +1005,7 @@ static void setPeriod(moduleChannel_t *ch)
 			ch->n_replen = 1;
 		}
 
+		// set voice period
 		paulaWriteWord(voiceAddr + 6, ch->n_period);
 
 		DMACONtemp |= ch->n_dmabit;
@@ -1018,6 +1050,7 @@ static void playVoice(moduleChannel_t *ch)
 {
 	if (ch->n_note == 0 && ch->n_cmd == 0) // test period, command and command parameter
 	{
+		// set voice period
 		const uint32_t voiceAddr = 0xDFF0A0 + (ch->n_chanindex * 16);
 		paulaWriteWord(voiceAddr + 6, ch->n_period);
 
@@ -1099,10 +1132,10 @@ static void updateUIPositions(void)
 		return; // don't update UI under MOD2WAV/PAT2SMP rendering
 
 	song->currRow = song->row;
-	song->currOrder = modOrder;
+	song->currPos = modPos;
 	song->currPattern = modPattern;
 
-	uint16_t *currPatPtr = &song->header.order[modOrder];
+	uint16_t *currPatPtr = &song->header.patternTable[modPos];
 	editor.currPatternDisp = currPatPtr;
 	editor.currPosEdPattDisp = currPatPtr;
 	editor.currPatternDisp = currPatPtr;
@@ -1162,18 +1195,18 @@ static void nextPosition(void)
 			return;
 		}
 
-		modOrder = (modOrder + 1) & 127;
-		if (modOrder >= song->header.numOrders)
+		modPos = (modPos + 1) & 127;
+		if (modPos >= song->header.songLength)
 		{
-			modOrder = 0;
+			modPos = 0;
 
 			if (config.compoMode) // stop song for music competitions playing
 			{
 				doStopIt(true);
 				turnOffVoices();
 
-				modOrder = 0;
-				modPattern = (int8_t)song->header.order[modOrder];
+				modPos = 0;
+				modPattern = (int8_t)song->header.patternTable[modPos];
 				song->row = 0;
 
 				updateUIPositions();
@@ -1183,7 +1216,7 @@ static void nextPosition(void)
 				modRenderDone = true;
 		}
 
-		modPattern = (int8_t)song->header.order[modOrder];
+		modPattern = (int8_t)song->header.patternTable[modPos];
 		if (modPattern > MAX_PATTERNS-1)
 			modPattern = MAX_PATTERNS-1;
 	}
@@ -1210,7 +1243,7 @@ static void increasePlaybackTimer(void)
 static void setCurrRowToVisited(void) // for MOD2WAV
 {
 	if (editor.mod2WavOngoing)
-		editor.rowVisitTable[(modOrder * MOD_ROWS) + song->row] = true;
+		editor.rowVisitTable[(modPos * MOD_ROWS) + song->row] = true;
 }
 
 static bool renderEndCheck(void) // for MOD2WAV/PAT2SMP
@@ -1229,7 +1262,7 @@ static bool renderEndCheck(void) // for MOD2WAV/PAT2SMP
 
 		if (editor.mod2WavOngoing && song->tick == song->speed-1)
 		{
-			bool rowVisited = editor.rowVisitTable[(modOrder * MOD_ROWS) + song->row];
+			bool rowVisited = editor.rowVisitTable[(modPos * MOD_ROWS) + song->row];
 			if (rowVisited || modRenderDone)
 			{
 				modRenderDone = false;
@@ -1248,7 +1281,8 @@ static void setDMA(void)
 	if (editor.muted[2]) DMACONtemp &= ~4;
 	if (editor.muted[3]) DMACONtemp &= ~8;
 
-	paulaWriteWord(0xDFF096, 0x8000 | DMACONtemp); // start DMAs for selected voices
+	// start DMAs for selected voices
+	paulaWriteWord(0xDFF096, 0x8000 | DMACONtemp);
 
 	setVisualsDMACON(0x8000 | DMACONtemp);
 
@@ -1258,7 +1292,7 @@ static void setDMA(void)
 		if (DMACONtemp & ch->n_dmabit) // handle visuals on sample trigger
 			setVUMeterHeight(ch);
 
-		// these take effect after the current DMA cycle is done
+		// set new voice data ptr and length (these take effect after the current DMA cycle is done)
 		const uint32_t voiceAddr = 0xDFF0A0 + (i * 16);
 		paulaWritePtr(voiceAddr + 0, ch->n_loopstart);
 		paulaWriteWord(voiceAddr + 4, ch->n_replen);
@@ -1334,6 +1368,7 @@ bool intMusic(void) // replayer ticker
 			{
 				playVoice(ch);
 
+				// set voice volume
 				const uint32_t voiceAddr = 0xDFF0A0 + (i * 16);
 				paulaWriteWord(voiceAddr + 8, ch->n_volume);
 
@@ -1454,7 +1489,7 @@ void modSetPattern(uint8_t pattern)
 	ui.updateCurrPattText = true;
 }
 
-void modSetPos(int16_t order, int16_t row)
+void modSetPos(int16_t pos, int16_t row)
 {
 	if (row != -1)
 	{
@@ -1465,17 +1500,16 @@ void modSetPos(int16_t order, int16_t row)
 		song->currRow = (int8_t)row;
 	}
 
-	if (order != -1)
+	if (pos != -1)
 	{
-		if (order >= 0)
+		if (pos >= 0)
 		{
-			modOrder = order;
-			song->currOrder = order;
+			song->currPos = modPos = pos;
 			ui.updateSongPos = true;
 
 			if (editor.currMode == MODE_PLAY && editor.playMode == PLAY_MODE_NORMAL)
 			{
-				modPattern = (int8_t)song->header.order[order];
+				modPattern = (int8_t)song->header.patternTable[pos];
 				if (modPattern > MAX_PATTERNS-1)
 					modPattern = MAX_PATTERNS-1;
 
@@ -1484,13 +1518,13 @@ void modSetPos(int16_t order, int16_t row)
 			}
 
 			ui.updateSongPattern = true;
-			editor.currPatternDisp = &song->header.order[modOrder];
+			editor.currPatternDisp = &song->header.patternTable[modPos];
 
-			int16_t posEdPos = song->currOrder;
-			if (posEdPos > song->header.numOrders-1)
-				posEdPos = song->header.numOrders-1;
+			int16_t posEdPos = song->currPos;
+			if (posEdPos > song->header.songLength-1)
+				posEdPos = song->header.songLength-1;
 
-			editor.currPosEdPattDisp = &song->header.order[posEdPos];
+			editor.currPosEdPattDisp = &song->header.patternTable[posEdPos];
 
 			if (ui.posEdScreenShown)
 				ui.updatePosEd = true;
@@ -1584,7 +1618,7 @@ void decPatt(void)
 	ui.updateCurrPattText = true;
 }
 
-void modPlay(int16_t patt, int16_t order, int8_t row)
+void modPlay(int16_t patt, int16_t pos, int8_t row)
 {
 	const bool audioWasntLocked = !audio.locked;
 	if (audioWasntLocked)
@@ -1609,32 +1643,23 @@ void modPlay(int16_t patt, int16_t order, int8_t row)
 
 	if (editor.playMode != PLAY_MODE_PATTERN)
 	{
-		if (modOrder >= song->header.numOrders)
-		{
-			modOrder = 0;
-			song->currOrder = 0;
-		}
+		if (modPos >= song->header.songLength)
+			song->currPos = modPos = 0;
 
-		if (order >= 0 && order < song->header.numOrders)
-		{
-			modOrder = order;
-			song->currOrder = order;
-		}
+		if (pos >= 0 && pos < song->header.songLength)
+			song->currPos = modPos = pos;
 
-		if (order >= song->header.numOrders)
-		{
-			modOrder = 0;
-			song->currOrder = 0;
-		}
+		if (pos >= song->header.songLength)
+			song->currPos = modPos = 0;
 	}
 
 	if (patt >= 0 && patt <= MAX_PATTERNS-1)
 		song->currPattern = modPattern = (int8_t)patt;
 	else
-		song->currPattern = modPattern = (int8_t)song->header.order[modOrder];
+		song->currPattern = modPattern = (int8_t)song->header.patternTable[modPos];
 
-	editor.currPatternDisp = &song->header.order[modOrder];
-	editor.currPosEdPattDisp = &song->header.order[modOrder];
+	editor.currPatternDisp = &song->header.patternTable[modPos];
+	editor.currPosEdPattDisp = &song->header.patternTable[modPos];
 
 	song->tick = song->speed-1;
 	ciaSetBPM = -1; // fix possibly stuck "set BPM" flag
@@ -1671,9 +1696,11 @@ void clearSong(void)
 	if (song == NULL)
 		return;
 
-	memset(song->header.order, 0, sizeof (song->header.order));
+	memset(song->header.patternTable, 0, sizeof (song->header.patternTable));
 	memset(song->header.name, 0, sizeof (song->header.name));
 
+	posEdClearNames();
+
 	editor.muted[0] = false;
 	editor.muted[1] = false;
 	editor.muted[2] = false;
@@ -1694,7 +1721,7 @@ void clearSong(void)
 	editor.blockMarkFlag = false;
 	editor.swapChannelFlag = false;
 
-	song->header.numOrders = 1;
+	song->header.songLength = 1;
 
 	for (int32_t i = 0; i < MAX_PATTERNS; i++)
 		memset(song->patterns[i], 0, (MOD_ROWS * PAULA_VOICES) * sizeof (note_t));
@@ -1710,10 +1737,10 @@ void clearSong(void)
 
 	modSetPos(0, 0); // this also refreshes pattern data
 
-	song->currOrder = 0;
+	song->currPos = 0;
 	song->currPattern = 0;
-	editor.currPatternDisp = &song->header.order[0];
-	editor.currPosEdPattDisp = &song->header.order[0];
+	editor.currPatternDisp = &song->header.patternTable[0];
+	editor.currPosEdPattDisp = &song->header.patternTable[0];
 
 	modSetTempo(editor.initialTempo, true);
 	modSetSpeed(editor.initialSpeed);
@@ -1752,11 +1779,9 @@ void clearSamples(void)
 	editor.currSample = 0;
 	editor.hiLowInstr = 0;
 	editor.sampleZero = false;
-	ui.editOpScreenShown = false;
-	ui.aboutScreenShown = false;
 	editor.blockMarkFlag = false;
-
 	editor.samplePos = 0;
+
 	updateCurrSample();
 }
 
@@ -1799,7 +1824,7 @@ void restartSong(void) // for the beginning of MOD2WAV/PAT2SMP
 	song->currRow = 0;
 	song->rowsCounter = 0;
 
-	memset(editor.rowVisitTable, 0, MOD_ORDERS * MOD_ROWS); // for MOD2WAV
+	memset(editor.rowVisitTable, 0, 128 * MOD_ROWS); // for MOD2WAV
 
 	if (editor.pat2SmpOngoing)
 	{
@@ -1835,19 +1860,19 @@ void resetSong(void) // only call this after storeTempVariables() has been calle
 
 	initializeModuleChannels(song);
 
-	modOrder = oldOrder;
+	modPos = oldPos;
 	modPattern = (int8_t)oldPattern;
 
 	song->row = oldRow;
 	song->currRow = oldRow;
 	song->currBPM = oldBPM;
-	song->currOrder = oldOrder;
+	song->currPos = oldPos;
 	song->currPattern = oldPattern;
 
-	editor.currPosDisp = &song->currOrder;
+	editor.currPosDisp = &song->currPos;
 	editor.currEditPatternDisp = &song->currPattern;
-	editor.currPatternDisp = &song->header.order[song->currOrder];
-	editor.currPosEdPattDisp = &song->header.order[song->currOrder];
+	editor.currPatternDisp = &song->header.patternTable[song->currPos];
+	editor.currPosEdPattDisp = &song->header.patternTable[song->currPos];
 
 	modSetSpeed(oldSpeed);
 	modSetTempo(oldBPM, true);
