@@ -1,6 +1,10 @@
 /* Experimental audio sampling support.
 ** There may be several bad practices here, as I don't really
 ** have the proper knowledge on this stuff.
+**
+** Some functions like sin() may be different depending on
+** math library implementation, but we don't use pt2_math.c
+** replacements for speed reasons.
 */
 
 // for finding memory leaks in debug mode with Visual Studio 
@@ -11,20 +15,17 @@
 #include <stdio.h>
 #include <stdint.h>
 #include <stdbool.h>
-#include "pt2_header.h"
 #include "pt2_textout.h"
-#include "pt2_mouse.h"
-#include "pt2_structs.h"
-#include "pt2_sampler.h" // fixSampleBeep() / sampleLine()
+#include "pt2_sampler.h"
 #include "pt2_visuals.h"
 #include "pt2_helpers.h"
 #include "pt2_bmp.h"
-#include "pt2_unicode.h"
 #include "pt2_audio.h"
 #include "pt2_tables.h"
 #include "pt2_config.h"
-#include "pt2_sinc.h"
 #include "pt2_sampling.h"
+#include "pt2_math.h" // PT2_PI
+#include "pt2_replayer.h"
 
 enum
 {
@@ -36,13 +37,14 @@ enum
 // this may change after opening the audio input device
 #define SAMPLING_BUFFER_SIZE 1024
 
-#define FRAC_BITS 24
-#define FRAC_SCALE (1L << 24)
-#define FRAC_MASK (FRAC_SCALE-1)
+// after several tests, these values yields a good trade-off between quality and compute time
+#define SINC_TAPS 64
+#define SINC_TAPS_BITS 6 /* log2(SINC_TAPS) */
+#define SINC_PHASES 4096
+#define MID_TAP ((SINC_TAPS/2)*SINC_PHASES)
 
 #define SAMPLE_PREVIEW_WITDH 194
 #define SAMPLE_PREVIEW_HEIGHT 38
-
 #define MAX_INPUT_DEVICES 99
 #define VISIBLE_LIST_ENTRIES 4
 
@@ -50,14 +52,140 @@ static volatile bool callbackBusy, displayingBuffer, samplingEnded;
 static bool audioDevOpen;
 static char *audioInputDevs[MAX_INPUT_DEVICES];
 static uint8_t samplingNote = 33, samplingFinetune = 4; // period 124, max safe period for PAL Paula
-static int16_t displayBuffer[SAMPLING_BUFFER_SIZE], *bufferOrig, *buffer;
+static int16_t displayBuffer[SAMPLING_BUFFER_SIZE];
 static int32_t samplingMode = SAMPLE_MIX, inputFrequency, roundedOutputFrequency;
 static int32_t numAudioInputDevs, audioInputDevListOffset, selectedDev;
 static int32_t bytesSampled, maxSamplingLength, inputBufferSize;
-static float fOutputFrequency;
-static double dOutputFrequency;
+static double dOutputFrequency, *dSincTable, *dKaiserTable, *dSamplingBuffer, *dSamplingBufferOrig;
 static SDL_AudioDeviceID recordDev;
 
+/*
+** ----------------------------------------------------------------------------------
+** Sinc code taken from the OpenMPT project (has a similar BSD license), and modified
+** ----------------------------------------------------------------------------------
+*/
+
+static double Izero(double y) // Compute Bessel function Izero(y) using a series approximation
+{
+	double s = 1.0, ds = 1.0, d = 0.0;
+	const double epsilon = 1E-9; // 8bb: 1E-7 -> 1E-9 for added precision (still fast to calculate)
+
+	do
+	{
+		d = d + 2.0;
+		ds = ds * (y * y) / (d * d);
+		s = s + ds;
+	}
+	while (ds > epsilon * s);
+
+	return s;
+}
+
+bool initKaiserTable(void) // called once on tracker init
+{
+	dKaiserTable = (double *)malloc(SINC_TAPS * SINC_PHASES * sizeof (double));
+	if (dKaiserTable == NULL)
+	{
+		showErrorMsgBox("Out of memory!");
+		return false;
+	}
+
+	const double beta = 9.6377;
+	const double izeroBeta = Izero(beta);
+
+	for (int32_t i = 0; i < SINC_TAPS*SINC_PHASES; i++)
+	{
+		double fkaiser;
+		int32_t ix = (SINC_TAPS-1) - (i & (SINC_TAPS-1));
+
+		ix = (ix * SINC_PHASES) + (i >> SINC_TAPS_BITS);
+		if (ix == MID_TAP)
+		{
+			fkaiser = 1.0;
+		}
+		else
+		{
+			const double x = (ix - MID_TAP) * (1.0 / SINC_PHASES);
+			const double xMul = 1.0 / ((SINC_TAPS/2) * (SINC_TAPS/2));
+			fkaiser = Izero(beta * sqrt(1.0 - x * x * xMul)) / izeroBeta;
+		}
+
+		dKaiserTable[i] = fkaiser;
+	}
+
+	return true;
+}
+
+void freeKaiserTable(void)
+{
+	if (dKaiserTable != NULL)
+	{
+		free(dKaiserTable);
+		dKaiserTable = NULL;
+	}
+}
+
+// calculated after completion of sampling (before downsampling)
+static bool initSincTable(double cutoff)
+{
+	dSincTable = (double *)malloc(SINC_TAPS * SINC_PHASES * sizeof (double));
+	if (dSincTable == NULL)
+		return false;
+
+	if (cutoff > 1.0)
+		cutoff = 1.0;
+
+	const double kPi = PT2_PI * cutoff;
+	for (int32_t i = 0; i < SINC_TAPS*SINC_PHASES; i++)
+	{
+		double fsinc;
+		int32_t ix = (SINC_TAPS-1) - (i & (SINC_TAPS-1));
+
+		ix = (ix * SINC_PHASES) + (i >> SINC_TAPS_BITS);
+		if (ix == MID_TAP)
+		{
+			fsinc = 1.0;
+		}
+		else
+		{
+			const double x = (ix - MID_TAP) * (1.0 / SINC_PHASES);
+			const double xPi = x * kPi;
+
+			fsinc = (sin(xPi) / xPi) * dKaiserTable[i];
+		}
+
+		dSincTable[i] = fsinc * cutoff;
+	}
+
+	return true;
+}
+
+static void freeSincTable(void)
+{
+	if (dSincTable != NULL)
+	{
+		free(dSincTable);
+		dSincTable = NULL;
+	}
+}
+
+static double sinc(const double *dSmpData, const double dPhase)
+{
+	const int32_t phase = (int32_t)(dPhase * SINC_PHASES);
+	const double *dSincLUT = &dSincTable[phase << SINC_TAPS_BITS];
+
+	double dSmp = 0.0;
+	for (int32_t i = 0; i < SINC_TAPS; i++)
+		dSmp += dSmpData[i] * dSincLUT[i];
+
+	return dSmp;
+}
+
+/*
+** ----------------------------------------------------------------------------------
+** ----------------------------------------------------------------------------------
+*/
+
 static void listAudioDevices(void);
 
 static void updateOutputFrequency(void)
@@ -66,12 +194,11 @@ static void updateOutputFrequency(void)
 		samplingNote = 35;
 
 	int32_t period = periodTable[((samplingFinetune & 0xF) * 37) + samplingNote];
-	if (period < 113) // this happens internally in our Paula mixer
+	if (period < 113) // also happens in our "set period" Paula function
 		period = 113;
 
 	dOutputFrequency = (double)PAULA_PAL_CLK / period;
-	fOutputFrequency = (float)dOutputFrequency;
-	roundedOutputFrequency = (int32_t)(fOutputFrequency + 0.5f);
+	roundedOutputFrequency = (int32_t)(dOutputFrequency + 0.5);
 }
 
 static void SDLCALL samplingCallback(void *userdata, Uint8 *stream, int len)
@@ -83,7 +210,7 @@ static void SDLCALL samplingCallback(void *userdata, Uint8 *stream, int len)
 		if (len > SAMPLING_BUFFER_SIZE)
 			len = SAMPLING_BUFFER_SIZE;
 
-		const int16_t *L = (int16_t *)stream;
+		const int16_t *L =  (int16_t *)stream;
 		const int16_t *R = ((int16_t *)stream) + 1;
 
 		int16_t *dst16 = displayBuffer;
@@ -116,22 +243,22 @@ static void SDLCALL samplingCallback(void *userdata, Uint8 *stream, int len)
 		const int16_t *L = (int16_t *)stream;
 		const int16_t *R = ((int16_t *)stream) + 1;
 
-		int16_t *dst16 = &buffer[bytesSampled];
+		double *dSmp = &dSamplingBuffer[bytesSampled];
 
 		if (samplingMode == SAMPLE_LEFT)
 		{
 			for (int32_t i = 0; i < len; i++)
-				dst16[i] = L[i << 1];
+				dSmp[i] = L[i << 1] * (1.0 / 32768.0);
 		}
 		else if (samplingMode == SAMPLE_RIGHT)
 		{
 			for (int32_t i = 0; i < len; i++)
-				dst16[i] = R[i << 1];
+				dSmp[i] = R[i << 1] * (1.0 / 32768.0);
 		}
 		else
 		{
 			for (int32_t i = 0; i < len; i++)
-				dst16[i] = (L[i << 1] + R[i << 1]) >> 1;
+				dSmp[i] = (L[i << 1] + R[i << 1]) * (1.0 / (32768.0 * 2.0));
 		}
 
 		bytesSampled += len;
@@ -153,6 +280,7 @@ static void stopInputAudio(void)
 		SDL_CloseAudioDevice(recordDev);
 		recordDev = 0;
 	}
+	callbackBusy = false;
 }
 
 static void startInputAudio(void)
@@ -187,12 +315,6 @@ static void startInputAudio(void)
 	SDL_PauseAudioDevice(recordDev, false);
 }
 
-static void changeStatusText(const char *text)
-{
-	fillRect(88, 127, 17*FONT_CHAR_W, FONT_CHAR_H, video.palette[PAL_GENBKG]);
-	textOut(88, 127, text, video.palette[PAL_GENTXT]);
-}
-
 static void selectAudioDevice(int32_t dev)
 {
 	if (dev < 0)
@@ -208,8 +330,6 @@ static void selectAudioDevice(int32_t dev)
 		return;
 
 	listAudioDevices();
-	changeStatusText("PLEASE WAIT ...");
-	flipFrame();
 
 	stopInputAudio();
 	selectedDev = dev;
@@ -353,6 +473,10 @@ static void showCurrSample(void)
 
 void renderSamplingBox(void)
 {
+	changeStatusText("PLEASE WAIT ...");
+	flipFrame();
+	hpc_ResetCounters(&video.vblankHpc);
+
 	editor.sampleZero = false;
 	editor.blockMarkFlag = false;
 
@@ -380,8 +504,8 @@ void renderSamplingBox(void)
 	selectAudioDevice(selectedDev);
 
 	showCurrSample();
-
 	modStop();
+
 	editor.songPlaying = false;
 	editor.playMode = PLAY_MODE_NORMAL;
 	editor.currMode = MODE_IDLE;
@@ -414,7 +538,7 @@ static uint8_t getDispBuffPeak(const int16_t *smpData, int32_t smpNum)
 
 void writeSampleMonitorWaveform(void) // called every frame
 {
-	if (!ui.samplingBoxShown || ui.askScreenShown)
+	if (!ui.samplingBoxShown || ui.askBoxShown)
 		return;
 
 	if (samplingEnded)
@@ -478,15 +602,19 @@ static void startSampling(void)
 
 	assert(roundedOutputFrequency > 0);
 
-	maxSamplingLength = (int32_t)(ceil((65534.0*inputFrequency) / dOutputFrequency)) + 1;
+	maxSamplingLength = (int32_t)(ceil(((double)config.maxSampleLength*inputFrequency) / dOutputFrequency)) + 1;
 	
-	bufferOrig = (int16_t *)calloc(SINC_TAPS + maxSamplingLength + SINC_TAPS, sizeof (int16_t));
-	if (bufferOrig == NULL)
+	const int32_t allocLen = (SINC_TAPS/2) + maxSamplingLength + (SINC_TAPS/2);
+	dSamplingBufferOrig = (double *)malloc(allocLen * sizeof (double));
+	if (dSamplingBufferOrig == NULL)
 	{
 		statusOutOfMemory();
 		return;
 	}
-	buffer = bufferOrig + SINC_TAPS; // allow negative look-up for sinc taps
+	dSamplingBuffer = dSamplingBufferOrig + (SINC_TAPS/2); // allow negative look-up for sinc taps
+
+	// clear tap area before sample
+	memset(dSamplingBufferOrig, 0, (SINC_TAPS/2) * sizeof (double));
 
 	bytesSampled = 0;
 	audio.isSampling = true;
@@ -498,111 +626,86 @@ static void startSampling(void)
 	setStatusMessage("SAMPLING ...", NO_CARRY);
 }
 
-static uint16_t downsampleSamplingBuffer(void)
+static int32_t downsampleSamplingBuffer(void)
 {
+	// clear tap area after sample
+	memset(&dSamplingBuffer[bytesSampled], 0, (SINC_TAPS/2) * sizeof (double));
+
 	const int32_t readLength = bytesSampled;
 	const double dRatio = dOutputFrequency / inputFrequency;
 	
 	int32_t writeLength = (int32_t)(readLength * dRatio);
-	if (writeLength > MAX_SAMPLE_LEN)
-		writeLength = MAX_SAMPLE_LEN;
+	if (writeLength > config.maxSampleLength)
+		writeLength = config.maxSampleLength;
 
-	//config.normalizeSampling = false;
-
-	double *dBuffer = NULL;
-	if (config.normalizeSampling)
+	double *dBuffer = (double *)malloc(writeLength * sizeof (double));
+	if (dBuffer == NULL)
 	{
-		dBuffer = (double *)malloc(writeLength * sizeof (double));
-		if (dBuffer == NULL)
-		{
-			statusOutOfMemory();
-			return 0;
-		}
-	}
-
-	const double dCutoff = dRatio * 0.97; // slightly below nyquist
-	if (!initSinc(dCutoff))
-	{
-		if (config.normalizeSampling)
-			free(dBuffer);
-
 		statusOutOfMemory();
-		return 0;
+		return -1;
 	}
 
-	changeStatusText("DOWNSAMPLING ...");
-	flipFrame();
+	if (!initSincTable(dRatio))
+	{
+		statusOutOfMemory();
+		return -1;
+	}
 
 	// downsample
 
 	int8_t *output = &song->sampleData[song->samples[editor.currSample].offset];
-
 	const double dDelta = inputFrequency / dOutputFrequency;
-	int16_t *smpPtr = &buffer[-((SINC_TAPS/2)-1)]; // pre-centered (this is safe, look at how bufferOrig is alloc'd)
 
-	double dFrac = 0.0;
-	if (config.normalizeSampling)
+	// pre-centered (this is safe, look at how dSamplingBufferOrig is alloc'd)
+	const double *dSmpPtr = &dSamplingBuffer[-((SINC_TAPS/2)-1)];
+
+	double dPhase = 0.0;
+	double dPeakAmp = 0.0;
+	for (int32_t i = 0; i < writeLength; i++)
 	{
-		double dPeakAmp = 0.0;
-		for (int32_t i = 0; i < writeLength; i++) // up to 65534 bytes
-		{
-			double dSmp = sinc(smpPtr, dFrac);
+		double dSmp = sinc(dSmpPtr, dPhase);
+		dBuffer[i] = dSmp;
 
-			dFrac += dDelta;
-			int32_t wholeSamples = (int32_t)dFrac;
-			dFrac -= wholeSamples;
-			smpPtr += wholeSamples;
+		// dSmp = fabs(dSmp)
+		if (dSmp < 0.0)
+			dSmp = -dSmp;
 
-			const double dAbsSmp = fabs(dSmp);
-			if (dAbsSmp > dPeakAmp)
-				dPeakAmp = dAbsSmp;
+		if (dSmp > dPeakAmp)
+			dPeakAmp = dSmp;
 
-			dBuffer[i] = dSmp;
-		}
-
-		// normalize
-
-		double dAmp = INT8_MAX / dPeakAmp;
-
-		/* If we have to amplify THIS much, it would mean that the gain was extremely low.
-		** We don't want to amplify a ton of noise, so keep it quantized to zero (silence).
-		*/
-		const double dAmp_dB = 20.0*log10(dAmp);
-		if (dAmp_dB > 40.0)
-			dAmp = 0.0;
-
-		for (int32_t i = 0; i < writeLength; i++)
-		{
-			/* To round the sample is probably incorrect, but it aliases audibly
-			** less after sampling a 1kHz sine wave, so I'll stick with it for now.
-			** Also just a note: Dithering is not very suitable for samples being
-			** played at lower pitches, hence why I don't dithering.
-			*/
-			const double dSmp = dBuffer[i] * dAmp;
-			int32_t smp32 = (int32_t)round(dSmp);
-			output[i] = (int8_t)CLAMP(smp32, -128, 127);
-		}
+		dPhase += dDelta;
+		const int32_t wholeSamples = (int32_t)dPhase;
+		dPhase -= wholeSamples;
+		dSmpPtr += wholeSamples;
 	}
-	else
+
+	freeSincTable();
+
+	// normalize
+
+	double dAmp = INT8_MAX / dPeakAmp;
+
+	/* If we have to amplify THIS much, it would mean that the gain was extremely low.
+	** We don't want the result to be 99% noise, so keep it quantized to zero (silence).
+	*/
+	const double dAmp_dB = 20.0 * log10(dAmp / 128.0);
+	if (dAmp_dB > 50.0)
+		dAmp = 0.0;
+
+	for (int32_t i = 0; i < writeLength; i++)
 	{
-		for (int32_t i = 0; i < writeLength; i++) // up to 65534 bytes
-		{
-			const double dSmp = sinc(smpPtr, dFrac);
-			int32_t smp32 = (int32_t)round(dSmp);
-			output[i] = (int8_t)CLAMP(smp32, -128, 127);
+		double dSmp = dBuffer[i] * dAmp;
 
-			dFrac += dDelta;
-			int32_t wholeSamples = (int32_t)dFrac;
-			dFrac -= wholeSamples;
-			smpPtr += wholeSamples;
-		}
+		// faster than calling round()
+		     if (dSmp < 0.0) dSmp -= 0.5;
+		else if (dSmp > 0.0) dSmp += 0.5;
+		const int32_t smp32 = (int32_t)dSmp; // rounded
+
+		output[i] = (int8_t)smp32;
 	}
-	freeSinc();
 
-	if (config.normalizeSampling)
-		free(dBuffer);
-
-	return (uint16_t)writeLength;
+	free(dBuffer);
+	return writeLength;
 }
 
 void stopSampling(void)
@@ -611,26 +714,25 @@ void stopSampling(void)
 	audio.isSampling = false;
 
 	int32_t newLength = downsampleSamplingBuffer();
-	if (newLength == 0)
+	if (newLength == -1)
 		return; // out of memory
 
+	if (dSamplingBufferOrig != NULL)
+	{
+		free(dSamplingBufferOrig);
+		dSamplingBufferOrig = NULL;
+	}
+
 	moduleSample_t *s = &song->samples[editor.currSample];
-	s->length = (uint16_t)newLength;
+	s->length = newLength;
 	s->fineTune = samplingFinetune;
 	s->loopStart = 0;
 	s->loopLength = 2;
 	s->volume = 64;
 	fixSampleBeep(s);
 
-	if (bufferOrig != NULL)
-	{
-		free(bufferOrig);
-		bufferOrig = NULL;
-	}
-
 	pointerSetMode(POINTER_MODE_IDLE, DO_CARRY);
-	displayMsg("SAMPLING DONE !");
-	setMsgPointer();
+	statusAllRight();
 
 	showCurrSample();
 }
@@ -733,16 +835,31 @@ void handleSamplingBox(void)
 	if (!mouse.leftButtonPressed)
 		return;
 
-	if (audio.isSampling)
-		stopSampling();
-
 	mouse.lastSamplingButton = -1;
 	mouse.repeatCounter = 0;
 
+	if (audio.isSampling)
+	{
+		stopSampling();
+		return;
+	}
+
 	// check buttons
 	const int32_t mx = mouse.x;
 	const int32_t my = mouse.y;
 
+	if (mx >= 182 && mx <= 243 && my >= 0 && my <= 10) // STOP (main UI)
+	{
+		turnOffVoices();
+	}
+
+	if (mx >= 6 && mx <= 25 && my >= 124 && my <= 133) // EXIT (main UI)
+	{
+		ui.samplingBoxShown = false;
+		removeSamplingBox();
+		exitFromSam();
+	}
+
 	if (mx >= 98 && mx <= 108 && my >= 44 && my <= 54) // SAMPLE UP (main UI)
 	{
 		samplingSampleNumUp();
