@@ -53,17 +53,14 @@ void updateEditOp(void);
 void toggleFullscreen(void);
 void videoClose(void);
 void displayMainScreen(void);
-void renderPosEdScreen(void);
 void renderMuteButtons(void);
 void renderAboutScreen(void);
 void renderQuadrascopeBg(void);
 void renderSpectrumAnalyzerBg(void);
 void renderEditOpMode(void);
-void renderTextEditMarker(void);
 void renderEditOpScreen(void);
 void renderSamplerVolBox(void);
 void renderSamplerFiltersBox(void);
-void removeTextEditMarker(void);
 void removeSamplerVolBox(void);
 void removeSamplerFiltersBox(void);
 void fillToVuMetersBgBuffer(void);
